package figures;

import points.Point2D;

public class Cercle extends AbstractFigure {
	private double rayon;
	private Point2D centre;
	
	public Cercle() {
		this.centre = new Point2D(0,0);
		this.rayon = 2;
	}
	
	public Cercle(double x, double y, double rayon) {
		this.centre = new Point2D(x,y);
		this.rayon = 2;
	}
	
	public Cercle(Point2D p, double rayon) {
		this.centre = p;
		this.rayon = 2;
	}
	
	public Cercle(Cercle c) {
		this.rayon = c.rayon;
		this.centre = c.centre;
	}
	
	public void setRayon(double r) {
		this.rayon = r;
	}
	
	public double getRayon() {
		return this.rayon;
	}
	
	
	@Override
	public String getNom() {
		return "Cercle";
	}

	@Override
	public Figure deplace(double dx, double dy) {
		this.centre.deplace(dx, dy);
		return this;
	}

	@Override
	public boolean contient(Point2D p) {
		return (p.getX() - this.centre.getX())*(p.getX() - this.centre.getX()) + (p.getY() - this.centre.getY())*(p.getY() - this.centre.getY()) < this.rayon*this.rayon;
	}

	@Override
	public Point2D getCentre() {
		return this.centre;
	}

	@Override
	public double aire() {
		return this.rayon*this.rayon*Math.PI;
	}
	
	@Override
	public double distanceToCentreOf(Figure f) {
		return Point2D.distance(this.centre, f.getCentre());
	}
	
	@Override
	public String toString() {
		return this.getNom() + " : "+ this.centre.toString() +", r = " + this.rayon;
	}
	
	
	@Override
	public int hashCode() {
		return super.hashCode();
	}

	@Override
	protected boolean equals(Figure figure) {
		if (getClass().equals(figure.getClass()))
		{
			Cercle other = (Cercle) figure;
			boolean found = false;
			if (this.centre.equals(other.getCentre()) && this.rayon == other.getRayon())
			{
				found = true;
			}
			if (!found)
			{
				return false;
			}
			return true;
		}
		else
		{
			return false;
		}
	}
}
