package figures;

import java.util.ArrayList;

import points.Point2D;

public class Polygone extends AbstractFigure {

	ArrayList<Point2D> listePoints;
	
	
	private double minimum(double v1, double v2) {
		double res = v1;
		if(v1 > v2) {
			res = v2;
		}
		return res;
	}
	
	private double maximum(double v1, double v2) {
		double res = v1;
		if(v1 < v2) {
			res = v2;
		}
		return res;
	}
	
	public Polygone() {
		this.listePoints = new ArrayList<Point2D>();
	}
	
	public Polygone(Point2D p) {
		this.listePoints = new ArrayList<Point2D>();
		this.listePoints.add(p);
	}
	
	public Polygone(Point2D p1, Point2D p2) {
		this.listePoints = new ArrayList<Point2D>();
		this.listePoints.add(p1);
		this.listePoints.add(p2);
	}
	
	public Polygone(Polygone p) {
		this.listePoints = new ArrayList<Point2D>(p.listePoints);
	}
	
	public Polygone(ArrayList<Point2D> list) {
		this.listePoints = new ArrayList<Point2D>();
		for(int i=0; i<list.size(); i++) {
			this.listePoints.add(list.get(i));
		}
	}
	
	public void ajouter(double x, double y) {
		Point2D p = new Point2D(x, y);
		this.listePoints.add(p);
	}
	
	public void ajouter(Point2D p) {
		this.listePoints.add(p);
	}
	
	public boolean supprimer(Point2D p) {
		boolean res = false;
		if(this.listePoints.contains(p)) {
			res = true;
			this.listePoints.remove(p);
		}
		return res;
	}
	
	public void supprimer() {
		this.listePoints.remove(0);
	}
	
	@Override
	public String getNom() {
		return "Polygone";
	}

	@Override
	public Figure deplace(double dx, double dy) {
		for(Point2D point: this.listePoints) {
			point.deplace(dx, dy);
		}
		return this;
	}

	/*int pnpoly(int nvert, float *vertx, float *verty, float testx, float testy)
	{
	  int i, j, c = 0;
	  for (i = 0, j = nvert-1; i < nvert; j = i++) {
	    if ( ((verty[i]>testy) != (verty[j]>testy)) &&
	     (testx < (vertx[j]-vertx[i]) * (testy-verty[i]) / (verty[j]-verty[i]) + vertx[i]) )
	       c = !c;
	  }
	  return c;
	}*/
	
	@Override
	public boolean contient(Point2D p) {
		int i, j;
		boolean res = false;
		for(i=0, j=this.listePoints.size()-1; i<this.listePoints.size(); j=i++) {
			if( ((this.listePoints.get(i).getY()>p.getY()) != (this.listePoints.get(j).getY()>p.getY())) &&
					(p.getX() < (this.listePoints.get(j).getX()-this.listePoints.get(i).getX())*(p.getY()-this.listePoints.get(i).getY())/
					(this.listePoints.get(j).getY()-this.listePoints.get(i).getY()+this.listePoints.get(i).getX()))) {
				res = true;
			}
		}
		return res;
	}

	@Override
	public Point2D getCentre() {
		double xmax = 999999;
		double ymax = 999999;
		double xmin = -999999; 
		double ymin = -999999;
		for(Point2D point: this.listePoints) {
			xmax = maximum(xmax, point.getX());
			xmin = minimum(xmin, point.getX());
			ymax = maximum(ymax, point.getY());
			ymin = minimum(ymin, point.getY());
		}
		return new Point2D(xmin+(xmax-xmin)/2, ymin+(ymax-ymin)/2);
	}

	@Override
	public double aire() {
		double sum = 0;
		for(int i=0; i<this.listePoints.size()-1; i++) {
			sum += (this.listePoints.get(i).getX()- this.listePoints.get(0).getX())*(this.listePoints.get(i+1).getY()- this.listePoints.get(0).getY())
					-(this.listePoints.get(i+1).getX()- this.listePoints.get(0).getX())*(this.listePoints.get(i).getY()- this.listePoints.get(0).getY());
		}
		sum = 0.5*sum;
		return sum;
	}
	
	@Override
	public double distanceToCentreOf(Figure f) {
		return Point2D.distance(this.getCentre(), f.getCentre());
	}
	
	@Override
	public String toString() {
		String res = this.getNom() + " : ";
		for(int i=0; i<this.listePoints.size(); i++) {
			res+= this.listePoints.get(i).toString() + ", ";
		}
		res = res.substring(0, res.length()-2);
		System.out.println(res);
		return res;
	}
	
	@Override
	public boolean equals(Object o) {
		return false;
	}
	
	@Override
	public int hashCode() {
		return super.hashCode();
	}

	@Override
	protected boolean equals(Figure f) {
		// TODO Auto-generated method stub
		return false;
	}

}
