package figures;

import points.Point2D;

public class Rectangle extends AbstractFigure {

	private Point2D pmin;
	private Point2D pmax;
	
	private double minimum(double v1, double v2) {
		double res = v1;
		if(v1 > v2) {
			res = v2;
		}
		return res;
	}
	
	private double maximum(double v1, double v2) {
		double res = v1;
		if(v1 < v2) {
			res = v2;
		}
		return res;
	}
	
	public Rectangle() {
		this.pmin = new Point2D(0,0);
		this.pmax = new Point2D(1,1);
	}
	
	public Rectangle(double x1, double x2, double y1, double y2) {
		this.pmin = new Point2D(x1,y1);
		this.pmax = new Point2D(x2,y2);
	}
	
	public Rectangle(Point2D p1, Point2D p2) {
		this.pmin = p1;
		this.pmax = p2;
	}
	
	public Rectangle(Rectangle r) {
		this.pmin = r.getMin();
		this.pmax = r.getMax();
	}
	
	public Point2D getMin() {
		return this.pmin;
	}
	
	public Point2D getMax() {
		return this.pmax;
	}
	
	public double hauteur() {
		return Math.abs(pmin.getY()-pmax.getY());
	}
	
	public double largeur() {
		return Math.abs(pmin.getX()-pmax.getX());
	}
	@Override
	public String getNom() {
		return "Rectangle";
	}

	@Override
	public Figure deplace(double dx, double dy) {
		this.pmin.deplace(dx, dy);
		this.pmax.deplace(dx, dy);
		return this;
	}

	@Override
	public boolean contient(Point2D p) {
		boolean res = false;
		double xmax = maximum(pmin.getX(), pmax.getX());
		double xmin = minimum(pmin.getX(), pmax.getX());
		double ymax = maximum(pmin.getY(), pmax.getY());
		double ymin = minimum(pmin.getY(), pmax.getY());
		if(p.getX() > xmin && p.getX() < xmax && p.getY() > ymin && p.getY() < ymax) {
			res = true;
		}
		return res;
	}

	@Override
	public Point2D getCentre() {
		return new Point2D((pmin.getX()+pmax.getX())/2, (pmin.getY()+pmax.getY())/2);
	}

	@Override
	public double aire() {
		return hauteur()*largeur();
	}
	
	@Override
	public String toString() {
		return this.getNom() + " : "+ this.pmin.toString() +", " + this.pmax.toString();
	}
	
	@Override
	public double distanceToCentreOf(Figure f) {
		return Point2D.distance(this.getCentre(), f.getCentre());
	}
	
	@Override
	public int hashCode() {
		return super.hashCode();
	}

	@Override
	protected boolean equals(Figure figure) {
		if (getClass().equals(figure.getClass()))
		{
			Rectangle other = (Rectangle) figure;
			boolean found = false;
			if (this.pmin.equals(other.getMin()) && this.pmax.equals(other.getMax()) || this.pmin.equals(other.getMax()) && this.pmax.equals(other.getMin()))
			{
				found = true;
			}
			if (!found)
			{
				return false;
			}
			return true;
		}
		else
		{
			return false;
		}
	}
}
