package listes;

import java.nio.file.NotLinkException;
import java.util.AbstractCollection;
import java.util.Collection;
import java.util.Iterator;
import java.util.Objects;


public class CollectionListe<E> extends AbstractCollection<E> {
    private Liste<E> liste;

    public CollectionListe()
    {
        this.liste = new Liste<E>();
    }

    public CollectionListe(Collection<E> c)
    {
        this.liste = new Liste<E>();
        Iterator<E> i = c.iterator();
        while (i.hasNext())
        {
            this.add(i.next());
        }
    }

    public Iterator<E> iterator()
    {
        return this.liste.iterator();
    }

    public boolean isEmpty()
    {
        return (this.size() == 0);
    }

    public boolean add(E e)
    {
        this.liste.add(e);
        return true;
    }

    public int size()
    {
        return this.liste.size();
    }

    public boolean remove(Object elt)
    {
        return this.liste.remove(elt);
    }

    public boolean removeAll(Collection c)
    {
        if (c == null)
        {
            throw new NullPointerException();
        }
        boolean res = false;
        Iterator<E> i = c.iterator();
        while (i.hasNext())
        {
            Object o = i.next();
            if (this.remove(o))
            {
                res = true;
            }
        }
        return res;
    }

    public boolean contains(Object elt)
    {
        return this.liste.contains(elt);
    }

    public boolean containsAll(Collection c)
    {
        boolean res = true;
        if (c.size() > this.size())
        {
            res = false;
        }
        else
        {
            Iterator i = c.iterator();
            while (i.hasNext())
            {
                res = this.contains(i.next());
            }
        }
        return res;
    }

    public boolean retainAll(Collection c)
    {
        if (c == null)
        {
            throw new NullPointerException();
        }
        boolean res = false;
        Iterator i = this.iterator();
        while (i.hasNext())
        {
            Object o = i.next();
            if (!c.contains(o))
            {
                this.remove(o);
                res = true;
            }
        }
        return res;
    }

    public void clear()
    {
        this.liste.clear();
    }

    public boolean equals(Object o)
    {
        boolean res = true;
        if (!(o instanceof Collection))
        {
            res = false;
        }
        else
        {
            Collection c = (Collection) o;
            if (c.size() != this.size())
            {
                res = false;
            }
            else
            {
                Iterator i1 = c.iterator();
                Iterator i2 = this.iterator();
                while (res && i1.hasNext())
                {
                    if (!i1.next().equals(i2.next()))
                    {
                        res = false;
                    }
                }
            }
        }
        return res;
    }
    
    public int hashCode()
    {
    	/*final int prime = 31;
		int hash = 1;
		// On additionne les hashcode des lments de la liste
		for(E elem : this.liste) {
			hash = prime * hash + elem.hashCode();
		}
		return hash;*/
    	return this.liste.hashCode();
    }
}
