package listes;

import java.util.Iterator;
import java.util.ListIterator;
import java.util.NoSuchElementException;

public class Liste<E> implements IListe<E> {
	private Maillon<E> suivant;
	private Maillon<E> tete;
	
	public Liste() {
		this.setSuivant(null);
		this.tete = null;
	}
	
	public Liste(E elt) {
		this.setSuivant(null);
		this.tete = new Maillon<E>(elt);
	}
	
	public Liste(Liste<E> l) {
		for(E elem : l) {
			this.add(elem);
		}
	}
	
	// *************************** CLASSE MAILLON ************************
		private class Maillon<E> {
			private Maillon<E> suivant;
			private E donnee;
			
			public Maillon(E elt) {
				this.donnee = elt;
				this.suivant = null;
			}
			
			@SuppressWarnings("unused")
			public Maillon(E elt, Maillon<E> suivant) {
				this.donnee = elt;
				this.suivant = suivant;
			}
			
			@SuppressWarnings("unused")
			public Maillon<E> getSuivant() {
				return suivant;
			}
			public void setSuivant(Maillon<E> suivant) {
				this.suivant = suivant;
			}
			public E getDonnee() {
				return donnee;
			}
			@SuppressWarnings("unused")
			public void setDonnee(E donnee) {
				this.donnee = donnee;
			}
		}
		
		// ***************************FIN DE LA CLASSE MAILLON ************************
	
	@Override
	public Iterator<E> iterator() {
		return new ListeIterator<E>(this.tete);
	}
	
	public class ListeIterator<F> implements Iterator<F> {

		private Maillon<F> current;
		private Maillon<F> last;
		private Maillon<F> penultimate;
		private F element;
		boolean nextCalled;
		
		
		public ListeIterator(Maillon<F> tete) {
			this.current = tete;
			this.last = null;
			this.penultimate = null;
			this.nextCalled = false;
		}
		
		@Override
		public boolean hasNext() {
			boolean res = false;
            if (this.nextCalled && this.current != null && this.current.getSuivant() != null) {
                res = true;
            }
            if (!this.nextCalled && tete != null) {
                res = true;
            }
            return res;
		}

		@SuppressWarnings("unchecked")
		@Override
		public F next() {
			if (!this.nextCalled) {
                this.current = (Liste<E>.Maillon<F>) tete;
                if (tete != null) {
                    this.element = (F) tete.getDonnee();
                }
                else {
                    this.element = null;
                }
                this.nextCalled = true;
            }
            else {
                if (hasNext()) {
                    this.current = this.current.getSuivant();
                    this.element = this.current.getDonnee();
                }
                else {
                    throw new NoSuchElementException();
                }
            }
            return this.element;
			
		}
		
		@Override
		public void remove() {
			/*if(nextCalled) {
				next();
				last = null;
				penultimate.suivant = current;
				nextCalled = false;
			}
			else {
				throw new IllegalStateException("remove : next not called yet");

			}*/
			
		}
	}
	
	
	public void insert(E elt)
	{
		if(elt == null) {
			throw new NullPointerException();
		}
		else {
			Maillon<E> temp = new Maillon<E>(elt);
			temp.setSuivant(this.tete);
			this.tete = temp;
		}
	}
	
	public boolean insert(E elt, int index)
	{
		boolean res = false;
		Maillon<E> temp = this.tete;
		Maillon<E> precedent = null;
		Maillon<E> nouv = new Maillon<E>(elt);

		if(elt==null) {
			return res;
		}
		int i=0;
		
		while(i<index && temp != null) {
			precedent = temp;
			temp=temp.suivant;
			i++;
		}
		if(i==index) {
			res = true;
			if(i==0) {
				nouv.setSuivant(temp);
				this.tete = nouv;
			}
			else {
				if(temp!=null) {
					nouv.suivant = temp; // le nouveau se lie au suivant du temp (pour l'instant)
					precedent.suivant = nouv; // le temp se lie au nouveau pour reformer la chaine
				}
				else {
					precedent.suivant= nouv;
				}
				
			}
		}
		
		return res;
	}
	
	public void add(E elt)
	{
		if(elt == null) {
			throw new NullPointerException();
		}
		else {
			Iterator<E> it = this.iterator();
			int i=0;
			while(it.hasNext()) {
				i++;
				it.next();
			}
			insert(elt, i);
			/*Maillon<E> temp = this.tete;
			Maillon<E> nouv = new Maillon<E>(elt);
			
			while(temp.suivant != null) {
				temp=temp.suivant;
			}
			temp.suivant = nouv; // le temp se lie au nouveau pour reformer la chaine*/

		}
	}
	
	public boolean remove(Object elt) {
        boolean res = false;
        Maillon<E> m = this.tete;
        if (this.tete != null && this.size() == 1)
        {
            this.tete = m.getSuivant();
            res = true;
        }
        else {
            boolean isFirstElement = false;
            if (elt != null && this.tete != null) {
                while (m.getSuivant() != null && !res) {
                    if (!m.getSuivant().getDonnee().equals(elt) && !m.getDonnee().equals(elt)) {
                        m = m.getSuivant();
                    } else {
                        if (m.getDonnee().equals(elt)) {
                            isFirstElement = true;
                        }
                        res = true;
                    }
                }
                if (res) {
                    if (isFirstElement) {
                        System.out.println(m.getDonnee());
                        this.tete = m.getSuivant();
                    } else {
                        m.setSuivant(m.getSuivant().getSuivant());
                    }
                }
 
            }
        }
        return res;
    }
	
	public boolean removeAll(E elt)
	{
		boolean res = false;
        if (elt != null && this.tete != null) {
            Maillon<E> m = this.tete;
            while (m.getSuivant() != null) {
                if (m == this.tete && m.getDonnee().equals(elt)) {
                    this.tete = m.getSuivant();
                    res = true;
                }
                else if (m.getSuivant().getDonnee().equals(elt)) {
                    m.setSuivant(m.getSuivant().getSuivant());
                    res = true;
                }
                else {
                    m = m.getSuivant();
                }
            }
        }
        return res;
	}
	
	public void clear()
	{
		/*Maillon<E> current = this.tete;
		
		while(current.suivant!=null) {
			Maillon<E> temp = current.suivant;
			current = null;
			current = temp;
		}*/
		tete = null;
	}
	
	public boolean empty()
	{
		return this.tete==null;
	}
	
	public int size()
	{
		int i=0;
		Iterator<E> it = this.iterator();
		while(it.hasNext()) {
			i++;
			it.next();
		}
		
		return i;
	}
	
	@Override
	public boolean equals(Object o)
	{
		if(o==null) {
			return false;
		}
		if (this.getClass().equals(o.getClass()))
		{
			
			@SuppressWarnings("unchecked")
			Liste<E> l = (Liste<E>) o;
			if(l.size() == this.size()) {
				Iterator<E> lIt = l.iterator();
				for(E elem : this)
				{
					E lElem = null;
					if(lIt.hasNext())   lElem =	lIt.next();
					if(!elem.equals(lElem))	return false;
				}
			}
			else {
				return false;
			}
			
			return true;
		}
		return false;
	}
	
	@Override
	public int hashCode()
	{
		final int prime = 31;
		int hash = 1;
		// On additionne les hashcode des lments de la liste
		for(E elem : this) {
			hash = prime * hash + elem.hashCode();
		}
		return hash;
	}
	
	@Override
	public String toString()
	{
		StringBuffer res = new StringBuffer();
		
		res.append("[");
		for(E elem : this) {
			res.append(elem.toString() + "->");
		}
		res.delete(res.length()-2, res.length());
		res.append("]");
		System.out.println(res);
		return res.toString();
	}
	public Maillon<E> getSuivant() {
		return suivant;
	}
	public void setSuivant(Maillon<E> suivant) {
		this.suivant = suivant;
	}

	public boolean contains(Object elt) {
		boolean res = false;
		for(E elem : this) {
			if(elem==elt) {
				res = true;
			}
		}
		return res;
	}
	
}
