package ensembles;

import java.util.Collection;

import tableaux.Tableau;

/**
 * Ensemble tri utilisant un {@link Tableau}
 *
 * @author davidroussel
 */
public class EnsembleTriTableau<E extends Comparable<E>> extends
		EnsembleTableau<E> implements EnsembleTri<E>
{

	/**
	 * Constructeur par dfaut d'un ensemble tri utilisant un {@link Tableau}
	 */
	public EnsembleTriTableau()
	{
		/*
		 * TODO Complter si besoin ...
		 */
		this.tableau = new Tableau<E>();
	}

	/**
	 * Constructeur de copie à partir d'un autre iterable
	 *
	 * @param elements l'itrable dont on veut copier les lments
	 */
	public EnsembleTriTableau(Iterable<E> elements)
	{
		this.tableau = new Tableau<E>();
		int rang;
		for(E elt : elements) {
			
			if(!this.contient(elt)) {
				rang = this.rang(elt);
				this.tableau.insertElement(elt, rang);
			}
			
		}
	}

	/**
	 * Ajout d'un lment de manière trie dans l'ensemble utilisant un
	 * {@link Tableau}
	 * @param element l'lment à ajouter de manière trie (on considèrera que
	 *            l'on ne peut pas ajouter d'lment null)
	 * @return true si l'lment n'tait pas djà prsent dans l'ensemble, false
	 *         sinon ou si l'on a tent d'insrer un lment null (auquel cas il
	 *         n'est pas insr).
	 * @see ensembles.EnsembleTableau#ajout(java.lang.Object)
	 * @see tableaux.Tableau#insertElement(E, int)
	 */
	@Override
	public boolean ajout(E element)
	{
		if(element==null) {
			return false;
		}
		else {
			if(this.contient(element)) {
				return false;
			}
			else {
				// ensemble.rang calcule la position de l'lment; l o il devra tre plac
				int rang = this.rang(element);
				this.tableau.insertElement(element, rang);
				return true;
			}
		}
	}

	/**
	 * Test d'galit d'un ensemble tri. Il est ncessaire de rimplmenter la
	 * comparaison avec un autre ensemble car l'ordre des lment aura son
	 * importance dans la comparaison ce qui n'tait pas le cas avec les
	 * ensembles non tris.
	 *
	 * @return true si l'objet obj est aussi un ensemble (pas forcment tri) et
	 *         qu'il contient exactement les mêmes lments dans le même ordre.
	 * @see ensembles.EnsembleGenerique#equals(java.lang.Object)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public boolean equals(Object obj)
	{
		/*
		 * TODO Remplacer par ...
		 * 	1 - obj == null ? ==> false
		 * 	2 - obj == this ? ==> true
		 * 	3 - obj est une instance de Ensemble<?>
		 * 		- caster obj en Ensemble<?>
		 * 		- si obj et this ont exactement les mêmes lments dans le
		 * 		  même ordre ==> true
		 * 		- sinon ==> false;
		 * 	  - sinon (obj n'est pas un Ensembble<?>) ==> false
		 */
		if(obj==null) {
			return false;
		}
		else if(obj == this) {
			return true;
		}
		else if(obj instanceof Ensemble<?>) {
			if(((Ensemble<?>)obj).typeElements() == this.typeElements()) {
				if(this.typeElements()==null) {
					return true;
				}
				else {
					boolean res = true;
					int i=0;
					for(E elt : (Ensemble<E>)obj) {
						if(!this.contient(elt)) {
							res = false;
						}
						else {
							/*if(((EnsembleTri<E>) obj).rang(elt) != this.rang(elt)) {
								res = false;
							}*/
							if(this.rang(elt) != i) {
								res = false;
							}
						}
						i++;
					}
					/*if(obj instanceof EnsembleTri<?>) {
						for(E elt : (EnsembleTri<E>)obj) {
							if(((EnsembleTri<E>) obj).rang(elt) != this.rang(elt)) {
								res = false;
							}
						}
					}
					else {
						res = false;
					}*/
					return res;
				}
			}
			else {
				return false;
			}
		}
		else {
			return false;
		}
	}

	/**
	 * Code de hachage d'un ensemble tri. Il est ncessaire de rimplmenter le
	 * code de hachage pour les ensembles tris car on considèrera que deux
	 * ensembles contenant les même lments mais dans des ordres diffrents
	 * seront eux-même diffrents. Il faut donc que la mthode hashCode prenne
	 * en compte l'ordre des lments (Comme dans les autres {@link Collection}
	 * d'ailleurs).
	 *
	 * @return le code de hachage de cet ensemble tri.
	 * @see tableaux.Tableau#hashCode() pour un exemple de hashage utilisant
	 *      l'ordre des lments
	 * @see ensembles.EnsembleGenerique#hashCode()
	 */
	@Override
	public int hashCode()
	{
		final int prime = 31;
		int result = 1;
		for (E elt : this)
		{
			result = (prime * result) + (elt == null ? 0 : elt.hashCode());
		}
		return result;
	}
}
